package org.fortiss.tooling.kernel.service;

import static org.fortiss.tooling.kernel.ToolingKernelActivator.getService;

import java.util.List;
import java.util.Set;
import java.util.function.Consumer;

import org.fortiss.tooling.kernel.ToolingKernelActivator;
import org.fortiss.tooling.kernel.model.constraints.IConstrained;
import org.fortiss.tooling.kernel.model.constraints.IConstraint;
import org.fortiss.tooling.kernel.model.constraints.IConstraintVerificationStatus;
import org.fortiss.tooling.kernel.utils.ServicesUtils;

/**
 * The verification service allows to register "constraint verifiers", e.g.,
 * a test suite runner, a temporal logic verifier, a WCET checker, a requirement review system.
 * 
 * @author aravantinos
 * @author $Author$
 * @version $Rev$
 * @ConQAT.Rating YELLOW Hash: 8AC9115D8F013A0A319B5D91FBF5C3BA
 */
public interface IConstraintVerificationService {
	/**
	 * Returns the service instance.
	 * 
	 * @deprecated use {@link ServicesUtils#getService(org.osgi.framework.Bundle, Class)} and
	 *             {@link ServicesUtils#ungetService(org.osgi.framework.Bundle, Class)}, instead.
	 *             See also {@link ToolingKernelActivator#getService(Class)} and
	 *             {@link ToolingKernelActivator#ungetService(Class)}
	 */
	@Deprecated
	/** Returns the service instance. */
	public static IConstraintVerificationService getInstance() {
		return getService(IConstraintVerificationService.class);
	}

	/**
	 * @param constraint
	 *            Verifies <code>constraint</code>. The framework always runs the verification in a
	 *            command. Should not yield any "useless" change because verification should always
	 *            update the constraint status and the checksums anyways.
	 */
	void verify(IConstraint constraint);

	/**
	 * @param constraint
	 * @return <code>true</code> if <code>constraint</code> is up to date.
	 */
	boolean isUpToDate(IConstraint constraint);

	/**
	 * @param status
	 * @return A list of possible automatic fixes for the (constraint of) <code>status</code>.
	 *         A status should be candidate to fixing if it is anything but successful or outdated.
	 */
	List<IFix> fixes(IConstraintVerificationStatus status);

	/** Interface for a "fix". */
	public interface IFix {
		/**
		 * Description of the fix. Useful to allow the user distinguish the fix in case there are
		 * several.
		 */
		String getDescription();

		/**
		 * @param status
		 *            Fixes (the constraint of) <code>status</code>.
		 */
		void runFix(IConstraintVerificationStatus status);
	}

	/** Utils. We store them to avoid having again thousands of different classes for one topic. */

	/** Base for fixes. */
	public static class FixBase implements IFix {
		/** The description. */
		private String description;

		/** The fix itself. */
		private Consumer<IConstraintVerificationStatus> fix;

		/** Constructor. */
		public FixBase(String description, Consumer<IConstraintVerificationStatus> fix) {
			this.description = description;
			this.fix = fix;
		}

		/** {@inheritDoc} */
		@Override
		public String getDescription() {
			return description;
		}

		/** {@inheritDoc} */
		@Override
		public void runFix(IConstraintVerificationStatus status) {
			fix.accept(status);
		}
	}

	/**
	 * @param constraint
	 * @param constrained
	 * @param index
	 *            Defines <code>constrained</code> as the <code>index</code>nth constrained model
	 *            element of <code>constraint</code>.
	 *            Leaves the other elements as they are if they exist, fill them with
	 *            <code>null</code> till <code>index</code> otherwise.
	 */
	public void setConstrained(IConstraint constraint, IConstrained constrained, int index);

	/**
	 * @return The registered classes of constraints.
	 */
	public Set<Class<?>> getConstraintTypes();
}
